;;; create-gc-request.scm

(define create-gc-set-type
  (make-set-type
   32  ; bit-size
   0  0 0 0 0  0 0 0 0     ; (most significant bits are unused)
			   ; DEFAULT VALUES
   'arc-mode               ; pie-slice
   'dashes                 ; 4 ([4,4])
   'dash-offset            ; 0
   'clip-mask              ; none
   'clip-y-origin          ; 0
   'clip-x-origin          ; 0
   'graphics-exposures     ; #t
   'subwindow-mode         ; clip-by-children
   'font                   ; server dependant
   'tile-stipple-y-origin  ; 0
   'tile-stipple-x-origin  ; 0
   'stipple                ; unspecified size pixmap full of ones
   'tile                   ; unspecified size pixmap full of fg pixel
   'fill-rule              ; solid
   'fill-style             ; even-odd
   'join-style             ; miter
   'cap-style              ; butt
   'line-style             ; solid
   'line-width             ; 0
   'background             ; 1
   'foreground             ; 0
   'plane-mask             ; all ones
   'function               ; copy
   ))

(define create-gc-components    ; this is ordered, ls bit to ms bit
  '(function plane-mask foreground background line-width line-style
    cap-style fill-style fill-rule tile stipple tile-stipple-x-origin
    tile-stipple-y-origin font subwindow-mode graphics-exposures
    clip-x-origin clip-y-origin clip-mask dash-offset dashes arc-mode))


(define function/fill!
  (lambda (str offset function)
    (number->card8/fill!
     str offset
     (case function
       ((clear)          0)
       ((and)            1)
       ((and-reverse)    2)
       ((copy)           3)
       ((and-inverted)   4)
       ((no-op)          5)
       ((xor)            6)
       ((or)             7)
       ((nor)            8)
       ((equiv)          9)
       ((invert)        10)
       ((or-reverse)    11)
       ((copy-inverted) 12)
       ((or-inverted)   13)
       ((nand)          14)
       ((set)           15)
       (else (error "unrecognized function" function))))))

(define line-style/fill!
  (lambda (str offset line-style)
    (number->card8/fill!
     str offset
     (case line-style
       ((solid)       0)
       ((on-off-dash) 1)
       ((double-dash) 2)
       (else (error "unrecognized line-style" line-style))))))

(define cap-style/fill!
  (lambda (str offset cap-style)
    (number->card8/fill!
     str offset
     (case cap-style
       ((not-last)   0)
       ((butt)       1)
       ((round)      2)
       ((projecting) 3)
       (else (error "unrecognized cap-style" cap-style))))))

(define join-style/fill!
  (lambda (str offset join-style)
    (number->card8/fill!
     str offset
     (case join-style
       ((miter  0))
       ((round) 1)
       ((bevel) 2)
       (else (error "unrecognized join-style" join-style))))))

(define fill-style/fill!
  (lambda (str offset fill-style)
    (number->card8/fill!
     str offset
     (case fill-style
       ((solid)           0)
       ((tiled)           1)
       ((stippled)        2)
       ((opaque-stippled) 3)
       (else (error "unrecognized fill-style" fill-style))))))

(define fill-rule/fill!
  (lambda (str offset fill-rule)
    (number->card8/fill!
     str offset
     (case fill-rule
       ((even-odd) 0)
       ((winding)  1)
       (else (error "unrecognized fill-rule" fill-rule))))))

(define subwindow/fill!
  (lambda (str offset subwindow)
    (number->card8/fill!
     str offset
     (case subwindow
       ((clip-by-children)  0)
       ((include-inferiors) 1)
       (else (error "unrecognized subwindow-mode" subwindow))))))

(define clip-mask/fill!
  (lambda (str offset clip-mask)
    (number->card8/fill!
     str offset
     (case clip-mask
       ((none) 0)
       (else clip-mask)))))

(define arc-mode/fill!
  (lambda (str offset arc-mode)
    (number->card8/fill!
     str offset
     (case arc-mode
       ((chord)     0)
       ((pie-slice) 1)
       (else (error "unrecognized arc-mode" arc-mode))))))


;;; returns a string containing the bitmask followed by the value-list

(define process-create-gc-attributes-alist
  (lambda (alist)
    (let ((set (let ((proto-set (make-set create-gc-set-type 0)))
		 (for-each (lambda (cell)
			     (set-bit proto-set (car cell)))
			   alist)
		 proto-set))
	  (bit-count (length alist)))
      ;; value-list size + bit-mask size
      (let ((str (make-string (+ (* 4 bit-count) 4) null-char)))
	(number->card32/fill! str 0 (set->number set))
	(let loop ((components create-gc-components) (offset 4))
	  (cond ((null? components)
		 (values str bit-count))
		((assq (car components) alist) =>
		 (lambda (component-cell)
		   (let ((component (car component-cell))
			 (value     (cadr component-cell)))
		     (case component
		       ((function)   (function/fill! str offset value))
		       ((plane-mask) (number->card32/fill! str offset value))
		       ((foreground) (number->card32/fill! str offset value))
		       ((background) (number->card32/fill! str offset value))
		       ((line-width) (number->card16/fill! str offset value))
		       ((line-style) (line-style/fill! str offset value))
		       ((cap-style)  (cap-style/fill!  str offset value))
		       ((join-style) (join-style/fill! str offset value))
		       ((fill-style) (fill-style/fill! str offset value))
		       ((fill-rule)  (fill-rule/fill!  str offset value))
		       ((tile)       (number->pixmap/fill! str offset value))
		       ((stipple)    (number->pixmap/fill! str offset value))
		       ((tile-stipple-x-origin) (number->int16/fill! str offset value))
		       ((tile-stipple-y-origin) (number->int16/fill! str offset value))
		       ((font)               (number->font/fill! str offset value))
		       ((subwindow-mode)     (subwindow/fill! str offset value))
		       ((graphics-exposures) (boolean->bool/fill! str offset value))
		       ((clip-x-origin) (number->int16/fill! str offset value))
		       ((clip-y-origin) (number->int16/fill! str offset value))
		       ((clip-mask)   (clip-mask/fill!      str offset value))
		       ((dash-offset) (number->card16/fill! str offset value))
		       ((dashes)      (number->card8/fill! str offset value))
		       ((arc-mode)    (arc-mode/fill!      str offset value))
		       (else (error "unrecognized component" component)))
		     ;; value-list is a tetrabyte array
		     (loop (cdr components) (+ offset 4)))))
		(else (loop (cdr components) offset))))))))


;;;;;;;;;;;;;;;;
;;; CreateGC ;;;
;;;;;;;;;;;;;;;;

(define create-gc
  (let ((opcode 55)
	(lock (make-lock))
	(str (make-string 12)))
    (number->card8/fill! str 0 opcode)
    (number->card8/fill! str 1 0)  ; unused
    (lambda (conn drawable attributes-alist)
      (bind ((bitmask&value-list n (process-create-gc-attributes-alist attributes-alist)))
	(let ((xserver (connection-xserver conn))
	      (gc-id (new-id))
	      (request-length (+ 4 n)))
	  (with-lock lock
            (lambda ()
	      (number->card16/fill! str 2 request-length)
	      (number->card32/fill! str 4 gc-id)
	      (number->drawable/fill! str 8 drawable)
	      (send xserver str bitmask&value-list)
	      gc-id)))))))


;;;;;;;;;;;;;;;;
;;; ChangeGC ;;;
;;;;;;;;;;;;;;;;

(define change-gc
  (let ((opcode 56)
	(lock (make-lock))
	(str (make-string 8)))
    (number->card8/fill! str 0 opcode)
    (number->card8/fill! str 1 0)  ; unused
    (lambda (conn gc attributes-alist)
      (bind ((bitmask&value-list n (process-create-gc-attributes-alist attributes-alist)))
	(let ((xserver (connection-xserver conn))
	      (request-length (+ 3 n)))
	  (with-lock lock
            (lambda ()
	      (number->card16/fill! str 2 request-length)
	      (number->card32/fill! str 4 gc)
	      (send xserver str bitmask&value-list))))))))
