;;; test.scm


;;;;;;;;;;;;;;;;;;;;;;;;
;;; Test alloc-color ;;;
;;;;;;;;;;;;;;;;;;;;;;;;

(define alloc-color-test
  (lambda ()
    (with-connection
     (lambda (connection)
       (let* ((colormap (default-colormap connection))
	      (reply    (alloc-color connection colormap #xffff 0 0)))
	 (display-alloc-color-reply reply))))))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Test alloc-color-cells ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;; I'm not really sure what this does. :(

(define alloc-color-cells-test
  (lambda ()
    (with-connection
     (lambda (connection)
       (let* ((colormap (default-colormap connection))
	      (number-of-colors 4)
	      (number-of-planes 2))
	 (display-alloc-color-cells-reply
	  (alloc-color-cells connection colormap number-of-colors number-of-planes #t)))))))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Test alloc-color-planes ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;; Don't know what this one does, either, but it's implemented. :)

(define alloc-color-planes-test
  (lambda ()
    (with-connection
     (lambda (connection)
       (let* ((colormap (default-colormap connection))
	      (number-of-colors 4)
	      (number-of-reds   1)
	      (number-of-greens 2)
	      (number-of-blues  1))
	 (display-alloc-color-planes-reply
	  (alloc-color-planes connection colormap number-of-colors
			      number-of-reds number-of-greens number-of-blues #t)))))))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Test alloc-named-color ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define alloc-named-color-test
  (lambda ()
    (with-connection
     (lambda (connection)
       (let* ((colormap (default-colormap connection))
	      (reply    (alloc-named-color connection colormap "green")))
	 (display-alloc-named-color-reply reply))))))


;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Test allow-events ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;

;;; don't know what this is, really.


;;;;;;;;;;;;;;;;;
;;; Test bell ;;;
;;;;;;;;;;;;;;;;;

(define bell-test
  (lambda ()
    (with-connection
     (lambda (connection)
       (let ((f (lambda (p)
		  (format #t "bell at ~a%~%" p)
		  (bell connection p)
		  (unix-sleep 3))))
	 (for-each f '(-100 -50 0 50 100))
	 )))))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; change-active-pointer-grab ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;; untested


;;;;;;;;;;;;;;;;;;;;
;;; change-hosts ;;;
;;;;;;;;;;;;;;;;;;;;

;;; untested


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Test make-connection and close-connection ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define connect-test
  (lambda ()
    (let ((conn (make-connection)))
      (display-connection conn)
      (close-connection conn))))


;;;;;;;;;;;;;;;;;;;;;;
;;; Test create-gc ;;;
;;;;;;;;;;;;;;;;;;;;;;

(define create-gc-test
  (lambda ()
    (with-connection
     (lambda (connection)
       (let ((root (root-window connection)))
	 (create-gc connection root '()))))))


;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Test create-window ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;

(define create-window-test
  (lambda ()
    (with-connection
     (lambda (connection)
       (let ((root   (root-window connection))
	     (depth  (root-depth connection))
	     (visual (root-visual connection)))
	 (let ((window (create-window connection root
				      0 0  100 100
				      1 depth 'input-output visual
				      '())))
	   window))))))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Test get-keyboard-mapping ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define get-keyboard-mapping-test
  (lambda ()
    (with-connection
     (lambda (connection)
       (let ((first-keycode (connection:min-keycode connection))
	     (last-keycode  (connection:max-keycode connection)))
	 (display-get-keyboard-mapping-reply
	  (get-keyboard-mapping connection
				first-keycode
				(+ (- last-keycode first-keycode) 1))))))))


;;;;;;;;;;;;;;;;;;;;;;;;
;;; Test image-text8 ;;;
;;;;;;;;;;;;;;;;;;;;;;;;

(define image-text8-test
  (lambda ()
    (with-connection
     (lambda (connection)
       (let ((root    (root-window connection))
	     (depth   (root-depth connection))
	     (white   (white-pixel connection))
	     (black   (black-pixel connection)))
	 (let* ((window (create-window connection root  0 0  100 100
				       1 depth 'copy-from-parent 'copy-from-parent
				       `((event-mask (exposure))
					 (background-pixel ,white))))
		(gc (create-gc connection window `((foreground ,black)
						   (background ,white)) )))
	   (format #t "Mapping Window~%")
	   (map-window connection window)
	   (let loop ((event (read-event connection)))
	     (format #t "The event we get: ~s~%" event)
	     (if (expose-event? event)
		 (begin (format #t "Image-Text8~%")
			(image-text8 connection window gc 50 50 "Hi")
			(unix-sleep 4))
		 (loop (read-event connection))))))))))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Test list-extensions ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define list-extensions-test
  (lambda ()
    (with-connection
     (lambda (connection)
       (display-list-extensions-reply
	(list-extensions connection))))))


;;;;;;;;;;;;;;;;;;;;;;;
;;; Test map-window ;;;
;;;;;;;;;;;;;;;;;;;;;;;

(define map-window-test
  (lambda ()
    (with-connection
     (lambda (connection)
       (let ((root  (root-window connection))
	     (depth (root-depth connection))
	     (white (white-pixel connection)))
	 (format #t "Creating Window~%")
	 (let ((window (create-window connection root  0 0  100 100
				      1 depth 'copy-from-parent 'copy-from-parent
				      `((event-mask (exposure))
					(background-pixel ,white)
					))))
	   (format #t "Mapping Window~%")
	   (map-window connection window)
	   (format #t "Waiting for expose~%")
	   (display-event (read-event connection))
	   (unix-sleep 4)))))))


;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Test no-operation ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;

(define no-operation-test
  (lambda ()
    (with-connection
     (lambda (connection)
       (no-operation connection)))))


;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Test poly-fill-arc ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;

(define poly-fill-arc-test
  (lambda ()
    (with-connection
     (lambda (connection)
       (let ((root    (root-window connection))
	     (depth   (root-depth  connection))
	     (white   (white-pixel connection))
	     (black   (black-pixel connection)))
         (let* ((window (create-window connection root  0 0  200 200
				       1 depth 'copy-from-parent 'copy-from-parent
				       `((event-mask (exposure))
					 (background-pixel ,white))))
		(gc (create-gc connection window '() )))
           (format #t "Mapping Window~%")
           (map-window connection window)
	   (format #t "Waiting for expose~%")
           (let loop ((event (read-event connection)))
             (if (expose-event? event)
                 (begin (format #t "PolyFillArc~%")
                        (poly-fill-arc
			 connection window gc
			 `((0 0 100 100 0 ,(* 64 360))
			   (125 25 50 50 0 ,(* 64 360))
			   (0 100 100 100 ,(* 64 90) ,(* 64 180))
			   (100 100 100 100 0 ,(* 64 45))))
			(unix-sleep 2))
                 (loop (read-event connection))))))))))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Test poly-fill-rectangle ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define poly-fill-rectangle-test
  (lambda ()
    (with-connection
     (lambda (connection)
       (let ((root  (root-window connection))
	     (depth (root-depth connection))
	     (white (white-pixel connection))
	     (black (black-pixel connection)))
         (let* ((window (create-window connection root  0 0  200 200
				       1 depth 'copy-from-parent 'copy-from-parent
				       `((event-mask (exposure button-press))
					 (background-pixel ,white))))
		(gc (create-gc connection window '() )))
           (format #t "Mapping Window~%")
           (map-window connection window)
	   (format #t "Waiting for expose~%")
	   (do ((event (read-event connection) (read-event connection)))
	       ((button-press-event? event) #t)
	     (if (expose-event? event)
                 (begin (format #t "PolyFillRectangle~%")
                        (poly-fill-rectangle
			 connection window gc
			 '((10 10 90 90) (20 120 60 60) (130 30 40 140))))
		 (display-event event)))))))))


;;;;;;;;;;;;;;;;;;;;;;
;;; Test poly-line ;;;
;;;;;;;;;;;;;;;;;;;;;;

(define poly-line-test
  (lambda ()
    (with-connection
     (lambda (connection)
       (let ((root    (root-window connection))
	     (depth   (root-depth connection))
	     (white   (white-pixel connection))
	     (black   (black-pixel connection)))
	 (let* ((window (create-window connection root  0 0  100 100
				       1 depth 'copy-from-parent 'copy-from-parent
				       `((event-mask (exposure))
					 (background-pixel ,white))))
		(gc (create-gc connection window '() )))
	   (format #t "Mapping Window~%")
	   (map-window connection window)
	   (format #t "Waiting for expose~%")
	   (let loop ((event (read-event connection)))
	     (if (expose-event? event)
		 (begin (format #t "PolyLine~%")
			(poly-line connection window gc 'origin
				   '(#(10 90) #(10 10) #(50 50) #(90 10) #(90 90)))
			(unix-sleep 2))
		 (loop (read-event connection))))))))))


;;;;;;;;;;;;;;;;;;;;;;;
;;; Test poly-point ;;;
;;;;;;;;;;;;;;;;;;;;;;;

(define poly-point-test
  (lambda ()
    (with-connection
     (lambda (connection)
       (let ((root    (root-window connection))
	     (depth   (root-depth connection))
	     (white   (white-pixel connection))
	     (black   (black-pixel connection)))
	 (let* ((window (create-window connection root  0 0  100 100
				       1 depth 'copy-from-parent 'copy-from-parent
				       `((event-mask (exposure))
					 (background-pixel ,white))))
		(gc (create-gc connection window '() )))
	   (format #t "Mapping Window~%")
	   (map-window connection window)
	   (format #t "Waiting for expose~%")
	   (let loop ((event (read-event connection)))
	     (if (expose-event? event)
		 (begin (format #t "PolyLine~%")
			(poly-point connection window gc 'origin
				    '(#(10 90) #(10 10) #(50 50) #(90 10) #(90 90)))
			(unix-sleep 2))
		 (loop (read-event connection))))))))))


;;;;;;;;;;;;;;;;;;;;;;;
;;; Test query-font ;;;
;;;;;;;;;;;;;;;;;;;;;;;

(define query-font-test
  (lambda ()
    (with-connection
     (lambda (connection)
       (let ((root (root-window connection)))
	 (let ((gc (create-gc connection root '() )))
	   (display-query-font-reply
	    (query-font connection gc))))))))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Test query-text-extents ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define query-text-extents-test
  (lambda ()
    (with-connection
     (lambda (connection)
       (let ((root (root-window connection)))
	 (let ((gc (create-gc connection root '() )))
	   (display-query-text-extents-reply
	    (query-text-extents connection gc (string->string16 "Hi")))))))))


;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Test Screen Saver ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;

(define screen-saver-test
  (lambda ()
    (with-connection
     (lambda (connection)
       (let ((orig-settings (get-screen-saver connection)))
	 (format #t "Original Screen Saver Settings~%")
	 (display-get-screen-saver-reply orig-settings)
	 (set-screen-saver connection 1 1 #f #t)
	 (format #t "New Settings~%")
	 (display-get-screen-saver-reply (get-screen-saver connection))
	 (unix-sleep 10)
	 (set-screen-saver connection
			   (get-screen-saver-reply:timeout orig-settings)
			   (get-screen-saver-reply:interval orig-settings)
			   (get-screen-saver-reply:prefer-blanking orig-settings)
			   (get-screen-saver-reply:allow-exposures orig-settings))
	 (format #t "Back to the original settings~%")
	 (display-get-screen-saver-reply (get-screen-saver connection)))))))
