;;;; tools.scm

;;;;;;;;;;;;;;
;;; Syntax ;;;
;;;;;;;;;;;;;;

;;; bind is a mess because of "Warning: segment matching not implemented" in scheme48

(define-syntax bind
  (syntax-rules ()
    ((bind () body ...)
     (begin body ...))
    ((bind ((var1 def)) body ...)
     (let ((var1 def)) body ...))
    ((bind ((var1 var2 def)) body ...)
     (call-with-values
      (lambda () def)
      (lambda (var1 var2) body ...)))
    ((bind ((var1 var2 var3 def)) body ...)
     (call-with-values
      (lambda () def)
      (lambda (var1 var2 var3) body ...)))
    ((bind ((var1 var2 var3 var4 def)) body ...)
     (call-with-values
      (lambda () def)
      (lambda (var1 var2 var3 var4) body ...)))
    ((bind ((var1 var2 var3 var4 var5 def)) body ...)
     (call-with-values
      (lambda () def)
      (lambda (var1 var2 var3 var4 var5) body ...)))
    ((bind ((var1 var2 var3 var4 var5 var6 def)) body ...)
     (call-with-values
      (lambda () def)
      (lambda (var1 var2 var3 var4 var5 var6) body ...)))
    ((bind ((var1 var2 var3 var4 var5 var6 var7 def)) body ...)
     (call-with-values
      (lambda () def)
      (lambda (var1 var2 var3 var4 var5 var6 var7) body ...)))
    ((bind ((var1 var2 var3 var4 var5 var6 var7 var8 def)) body ...)
     (call-with-values
      (lambda () def)
      (lambda (var1 var2 var3 var4 var5 var6 var7 var8) body ...)))
    ;; Whew.
    ((bind (vars-def morevars-def ...) body ...)
     (bind (vars-def)
	   (bind (morevars-def ...) body ...)))
    ))


;;; ;;; case using equal?
;;; ;;; Normal case uses eqv? (well, memv).
;;; 
;;; (define-syntax case-equal
;;;   (syntax-rules (else)
;;; 	((case-equal (key ...) clauses ...)
;;; 	 (let ((atom-key (key ...)))
;;; 	   (case-equal atom-key clauses ...)))
;;; 	((case-equal key (else result1 result2 ...))
;;; 	 (begin result1 result2 ...))
;;; 	((case-equal key ((atoms ...) result1 result2 ...))
;;; 	 (if (member key '(atoms ...))
;;; 	     (begin result1 result2 ...)))
;;; 	((case-equal key ((atoms ...) result1 result2 ...) clause clauses ...)
;;; 	 (if (member key '(atoms ...))
;;; 	     (begin result1 result2 ...)
;;; 	     (case-equal key clause clauses ...)))))

;;; ;;; Even better for my purposes: string-case
;;; 
;;; (define string-member
;;;   (lambda (item list)
;;; 	(cond ((null? list)	#f)
;;; 	      ((string=? item (car list)) #t)
;;; 	      (else (string-member item (cdr list))))))
;;; 
;;; (define-syntax string-case
;;;   (syntax-rules (else)
;;; 	((string-case (key ...) clauses ...)
;;; 	 (let ((string-key (key ...)))
;;; 	   (string-case string-key clauses ...)))
;;; 	((string-case key (else result1 result2 ...))
;;; 	 (begin result1 result2 ...))
;;; 	((string-case key ((strings ...) result1 result2 ...))
;;; 	 (if (string-member key '(strings ...))
;;; 	     (begin result1 result2 ...)))
;;; 	((string-case key ((strings ...) result1 result2 ...) clause clauses ...)
;;; 	 (if (string-member key '(strings ...))
;;; 	     (begin result1 result2 ...)
;;; 	     (string-case key clause clauses ...)))))


(define-syntax when
  (syntax-rules ()
    ((when condition body ...)
     (if condition
	 (begin body ...)
	 #f))))

(define-syntax unless
  (syntax-rules ()
    ((unless condition body ...)
     (if condition
	 #t
	 (begin body ...)))))

(define-syntax infinite-loop
  (syntax-rules ()
    ((infinite-loop body ...)
     (let loop () body ... (loop)))))


;;;;;;;;;;;;;;;;;;;;;
;;; General Tools ;;;
;;;;;;;;;;;;;;;;;;;;;

(define append!
  (lambda (list1 list2)
    (cond ((null? list1) list2)
	  ;;((not (pair? list1)) (error "expected list" list1))
	  ;;((not (pair? list2)) (error "expected list" list2))
	  (else (let loop ((list list1))
		  (if (null? (cdr list))
		      (begin (set-cdr! list list2)
			     list1)
		      (loop (cdr list))))))))

;;; (define every?
;;;   (lambda (predicate? args)
;;; 	(cond ((null? args) #t)
;;; 	      ((predicate? (car args)) (apply every? (cons predicate? (cdr args))))
;;; 	      (else #f))))
;;; 
;;; (define type-check
;;;   (lambda (predicate? error-message . args)
;;; 	(if (every? predicate? args)
;;; 	    #t
;;; 	    (error error-message args))))

(define string-map
  (lambda (f str)
    (let* ((len     (string-length str))
	   (new-str (make-string len)))
      (let loop ((i 0))
	(if (< i len)
	    (begin
	      (string-set! new-str i (f (string-ref str i)))
	      (loop (+ i 1)))
	    new-str)))))

(define identity
  (lambda (x) x))

;;; (define curry
;;;   (lambda (f val1)
;;; 	(lambda args
;;; 	  (apply f (cons val1 args)))))
;;; 
;;; (define rcurry
;;;   (lambda (f lval)
;;; 	(let ((lval-list (list lval)))
;;; 	  (lambda args
;;; 	    (apply f (append args lval-list))))))
;;; 
;;; (define flip-args
;;;   (lambda (f)
;;; 	(lambda (arg1 arg2)
;;; 	  (f arg2 arg1))))

(define first car)
(define second cadr)
(define third caddr)
(define fourth cadddr)
(define fifth  (lambda (x) (cadr (cdddr x)))) 
(define sixth  (lambda (x) (caddr (cdddr x)))) 

;;; (define map-cdr
;;;   (lambda (f alist)
;;; 	(map (lambda (x)
;;; 	       (f (cdr x)))
;;; 	     alist)))
;;; 
;;; (define map-cadr
;;;   (lambda (f alist)
;;; 	(map (lambda (x)
;;; 	       (f (cadr x)))
;;; 	     alist)))

(define repeat
  (lambda (n val)
    (if (zero? n)
	'()
	(cons val (repeat (- n 1) val)))))

(define repeat-fn
  (lambda (n f)
    (if (zero? n)
	'()
	(cons (f) (repeat-fn (- n 1) f)))))

;;; (define range
;;;   (lambda (m n)  ; (m n]
;;; 	(if (>= m n)
;;; 	    '()
;;; 	    (cons m (range (+ m 1) n)))))

(define vector-for-each
  (lambda (f v)
    (do ((i 0 (+ i 1)))
	((>= i (vector-length v)) #t)
      (f (vector-ref v i)))))

(define copy-string
  (lambda (str)
    (let* ((len (string-length str))
	   (new (make-string len)))
      (do ((i 1 (+ i 0)))
	  ((>= i len) new)
	(string-set! new i (string-ref str i))))))

(define symbol-append
  (lambda syms
    (string->symbol (apply string-append (map symbol->string syms)))))


;;;;;;;;;;;;;;;;;;;;;;
;;; Specific Tools ;;;
;;;;;;;;;;;;;;;;;;;;;;

;;; (gather number-to-read function-that-reads) => list of things read
;;; Read a bunch of objects

(define gather
  (lambda (number read-it)
    (let loop ((n number)
	       (pile '()))
      (if (= n 0)
	  pile
	  (loop (- n 1) (cons (read-it) pile))))))


;;; ;;; (arbitrary-data length)  => data
;;; ;;; return a piece of binary-data length bytes long
;;; 
;;; (define arbitrary-data
;;;   (lambda (len)
;;; 	(if (zero? len)
;;; 	    ""
;;; 	    (make-string len null-char))))


;;; (pad-length length-of-data)  => length of pad needed to land at 4 byte boundary

(define pad-length
  (lambda (n)
    (let ((r (remainder n 4)))
      (if (zero? r)
	  0
	  (- 4 r)))))


;;; (pad-data unpadded-data)  => (values length-of-unpadded-data pad-length padded-data)

(define pad-data
  (lambda (data)
    (let* ((n (string-length data))
	   (p (pad-length n))
	   (padded-data (string-append data (make-string p null-char))))
      (values n p padded-data))))


;;; ;;; (fill-data str)
;;; ;;;   => data stretched to fill 4 bytes with most significant bits set to zero
;;; 
;;; (define fill-data
;;;   (lambda (str)
;;; 	(let ((n (pad-length (string-length str))))
;;; 	  (string-append (make-string n null-char) str))))
